import logging
from os import environ

# Controlling the default logging level via environment variable
DEFAULT_LOGGING_LEVEL = environ.get("LOG_LEVEL", "INFO").upper()
# Set the default level for all libraries - including 3rd party packages
logging.basicConfig(level=DEFAULT_LOGGING_LEVEL)



from enum import Enum


class DatasetName(str, Enum):
    """
    The type of the dataset used for the model training
    """

    Imagenet = "ImageNet"
    CIFAR_10 = "CIFAR-10"
    Fashion_MNIST = "Fashion MNIST"
    COCO = "COCO"
    PASCAL_VOC = "PASCAL VOC"
    COCO_Segmentation = "COCO Segmentation"
    Cityscapes = "Cityscapes"
    Supervisely = "Supervisely"
    WoodScape = "WoodScape"
    WiderFace = "WiderFace"
    BDD100K = "BDD100K"
    OpenImage = "Open Image"
    LISA = "LISA Traffic Sign"
    DOTA = "DOTA"
    LFW = "LFW"
    CASIAWebFace = "CASIA-WebFace"
    Kinetics = "Kinetics"
    MaskedFaceNet = "MaskedFaceNet"
    ADE20K = "ADE20K"
    LVIS = "LVIS"
    Other = "Other"
    Empty = ""



from copy import deepcopy
from enum import Enum
from typing import Any, Dict, List

from deci_common.abstractions.base_model import Schema


class ErrorLevel(str, Enum):
    ERROR = "Error"
    WARNING = "Warning"


class ModelErrorRecord(Schema):
    """
    Serialized model error record. This is intended as an external interface to notify users of problems
        with models uploaded to their lab. The errors list is returned from get_model apis.
    """

    error_type: str
    level: ErrorLevel = ErrorLevel.ERROR
    params: Dict[str, Any]
    message: str = ""


class BaseModelError(Exception):
    """
    Base class for model errors. All errors derived from this class will be logged by deci's job workers
        as a ModelErrorRecord in the model's errors list.

    Note: the cause list for the error is iterated over, so it's Ok to `raise Exception(...) from ex`. The
        error that will be logged will be the first error inheriting from BaseModelError.
    Note: if no BaseModelError is found in the cause chain, the error is converted into ModelInternalError.
    Note: the magic happens in ModelErrorsInterface.
    """

    def __init__(self, user_friendly_message: str, **kwargs: "Any"):
        super().__init__(user_friendly_message)
        self._user_friendly_message = user_friendly_message

    def encode_error_record(self, level: "ErrorLevel" = ErrorLevel.ERROR) -> "ModelErrorRecord":
        error_type = type(self).__name__
        message = self._user_friendly_message
        params = deepcopy(self.__dict__)
        del params["_user_friendly_message"]
        return ModelErrorRecord(error_type=error_type, params=params, message=message, level=level)


class ModelInternalError(BaseModelError):
    """
    Internal error type for model errors. This will be logged in case an unhandled exception fails one of deci's
        job workers, but no BaseModelError exception was found in the cause chain.
    """

    def __init__(self) -> None:
        user_friendly_message = "An internal error occurred with the given model. Please contact Deci support."
        super().__init__(user_friendly_message)


class ModelInputDimsMismatchError(BaseModelError):
    def __init__(self, user_input_dims: "List[List[int]]", model_input_dims: "List[List[int]]"):
        user_friendly_message = (
            "Failed to benchmark the model using the user-provided input dimensions, "
            "which are different than the dimensions expected by the model."
        )
        super().__init__(user_friendly_message)
        self.user_input_dims = user_input_dims
        self.model_input_dims = model_input_dims


class InvalidInputDimensionsError(BaseModelError):
    def __init__(self, user_input_dims: "List[List[int]]", model_input_dims: "List[List[int]]"):
        user_friendly_message = (
            "The model has invalid input dimensions. Please edit the model and provide non-"
            "dynamic input dimensions (positive integers)"
        )
        super().__init__(user_friendly_message)
        self.user_input_dims = user_input_dims
        self.model_input_dims = model_input_dims


class UnsupportedStaticBatchSizeError(BaseModelError):
    def __init__(self, static_batch_size: int):
        user_friendly_message = (
            f"Failed to benchmark the model using a static batch size of {static_batch_size} which is not yet supported."
            f" Please try to upload a model with dynamic batch size."
        )
        super().__init__(user_friendly_message)
        self.static_batch_size = static_batch_size


class PrimaryBatchSizeAndStaticBatchSizeConflictError(BaseModelError):
    def __init__(self, static_batch_size: int, primary_batch_size: int):
        user_friendly_message = (
            "Benchmark process could not be resolved for the requested primary batch size because the model is set to different static batch size."
            "Please try to upload a model with dynamic batch size."
        )
        super().__init__(user_friendly_message)
        self.static_batch_size = static_batch_size
        self.primary_batch_size = primary_batch_size


class BadModelFileReason(str, Enum):
    CORRUPTED_ZIP_FILE = "Failed to unzip the uploaded zip file. Is the file corrupted?"
    BAD_SAVED_MODEL_ZIP = "The provided zip file doesn't contain a valid tensorflow2 saved model."


class BadModelFileError(BaseModelError):
    def __init__(self, reason: "BadModelFileReason", **extras: "Any"):
        user_friendly_message = reason.value
        super().__init__(user_friendly_message)

        self.reason = reason.name
        for param_name, param_value in extras.values():
            setattr(self, param_name, param_value)


class FailedToLoadModelError(BaseModelError):
    def __init__(self) -> None:
        user_friendly_message = "Failed to load the model with infery."
        super().__init__(user_friendly_message=user_friendly_message)


class ConversionToDynamicBatchSizeError(BaseModelError):
    def __init__(self) -> None:
        user_friendly_message = "Failed to convert the model from static batch size to dynamic batch size"
        super().__init__(user_friendly_message=user_friendly_message)


class ValidatingModelError(BaseModelError):
    def __init__(self) -> None:
        user_friendly_message = "Failed to validate model"
        super().__init__(user_friendly_message=user_friendly_message)



from enum import Enum
from typing import List, Type, Union

import numpy

from deci_common.abstractions.base_model import Schema
from deci_common.data_types.enum.deep_learning_task import DeepLearningTask
from deci_common.helpers import get_all_enum_fields_in_web_form


class BatchSize(int, Enum):
    ONE = 1
    EIGHT = 8
    SIXTEEN = 16
    THIRTY_TWO = 32
    SIXTY_FOUR = 64


class BatchSizeEdge(int, Enum):
    ONE = 1
    TWO = 2
    FOUR = 4
    EIGHT = 8


class Metric(str, Enum):
    ACCURACY = "accuracy"
    THROUGHPUT = "throughput"
    LATENCY = "latency"
    MEMORY = "memory_footprint"
    SIZE = "model_size"


class AccuracyMetricType(str, Enum):
    PERCENTAGE = "percentage"
    FLOAT = "float"


class AccuracyMetricKey(str, Enum):
    TOP1 = "Top-1"
    TOP5 = "Top-5"
    AUC = "AUC"
    PRECISION = "Precision"
    RECALL = "Recall"
    F1_SCORE = "F1 Score"
    TRUE_POSITIVES = "True Positives"
    TRUE_NEGATIVES = "True Negatives"
    FALSE_POSITIVES = "False Positives"
    FALSE_NEGATIVES = "False Negatives"
    MIOU = "mIoU"
    PIXEL_ACCURACY = "Pixel Accuracy"
    DICE_COEFFICIENT = "Dice Coefficient (F1 Score)"
    MAP = "mAP"
    COCO_MAP = "mAP@0.5:0.95"
    D1 = "D1"
    D2 = "D2"
    D3 = "D3"
    ABS_REL = "AbsRel"
    SQ_REL = "SqRel"
    RMSE = "Root Mean Squared Error"
    RMSElog = "Root Mean Squared Error - Log"
    SLLOG = "Sllog"
    LOG10 = "Log-10"
    PCK = "PCK"
    PCKH = "PCKh"
    PDJ = "PDJ"
    OKS = "OKS"
    CUSTOM = "Custom"


class MapDLTaskToDefaultAccuracyMetric(str, Enum):
    OBJECT_DETECTION = AccuracyMetricKey.COCO_MAP.value
    SEMANTIC_SEGMENTATION = AccuracyMetricKey.MIOU.value
    CLASSIFICATION = AccuracyMetricKey.TOP1.value


class AccuracyMetric(Schema):
    key: str
    value: float
    type: AccuracyMetricType
    isPrimary: bool


class Architecture(str, Enum):
    RESNET18 = "resnet18"
    RESNET50 = "resnet50"
    RESNET34 = "resnet34"
    RESNET50_3343 = "resnet50_3342"
    RESNET101 = "resnet101"
    RESNET152 = "resnet152"
    RESNET18_CIFAR = "resnet18_cifar"
    RESNET_CUSTOM = "resnet_custom"
    RESNET50_CUSTOM = "resnet50_custom"
    RESNET_CUSTOM_CIFAR = "resnet_custom_cifar"
    RESNET50_CUSTOM_CIFAR = "resnet50_custom_cifar"
    MOBILENET_V2 = "mobilenet_v2"
    MOBILENET_V2_135 = "mobilenet_v2_135"
    CUSTOM_MOBILENET_V2 = "mobilenet_v2_custom"
    MOBILENET_V3_Large = "mobilenet_v3_large"
    MOBILENET_V3_SMALL = "mobilenet_v3_small"
    MOBILEMET_V3_CUSTOM = "mobilenet_v3_custom"
    YOLO_V3 = "yolo_v3"
    TINY_YOLO_V3 = "tiny_yolo_v3"
    CUSTOM_DENSENET = "custom_densenet"
    DENSENET121 = "densenet121"
    DENSENET161 = "densenet161"
    DENSENET169 = "densenet169"
    DENSENET201 = "densenet201"
    SHELFNET18 = "shelfnet18"
    SHELFNET34 = "shelfnet54"
    SHELFNET50 = "shelfnet50"
    SHELFNET50_3343 = "shelfnet50_3343"
    SHELFNET101 = "shelfnet101"
    SHELFNET_V2_X0_5 = "shelfnet_v2_x0_5"
    SHELFNET_V2_X1_0 = "shelfnet_v2_x1_0"
    SHELFNET_V2_X1_5 = "shelfnet_v2_x1_5"
    SHELFNET_V2_X2_0 = "shelfnet_v2_x2_0"
    SHELFNET_V2_CUSTOM = "shelfnet_v2_custom"
    DARKNET53 = "darknet53"
    CSP_DARKENT53 = "CSPDarknet"
    RESNEXT50 = "resnext50"
    RESNEXT101 = "resnext101"
    GOOGLENET = "googlenet"
    EFFICIENTNET_B0 = "efficientnet_b0"
    EFFICIENTNET_B1 = "efficientnet_b1"
    EFFICIENTNET_B2 = "efficientnet_b2"
    EFFICIENTNET_B3 = "efficientnet_b3"
    EFFICIENTNET_B4 = "efficientnet_b4"
    EFFICIENTNET_B5 = "efficientnet_b5"
    EFFICIENTNET_B6 = "efficientnet_b6"
    EFFICIENTNET_B7 = "efficientnet_b7"
    EFFICIENTNET_B8 = "efficientnet_b8"
    EFFICIENTNET_L2 = "efficientnet_l2"
    EFFICIENTNET_CUSTOM = "efficientnet_custom"
    REGNETY200 = "regnety200"
    REGNETY400 = "regnety400"
    REGNETY600 = "regnety600"
    REGNETY800 = "regnety800"
    REGNET_CUSTOM = "regnet_custom"
    REGNET_NAS = "regnet_nas"
    YOLO_V5S = "yolo_v5s"
    YOLO_V5M = "yolo_v5m"
    YOLO_V5L = "yolo_v5l"
    YOLO_V5X = "yolo_v5x"
    YOLO_V5_CUSTOM = "yolo_v5_custom"
    YOLO_V5_MOBILE = "yolo_v5_mobile"
    SSD_MOBILENET_V1 = "ssd_mobilenet_v1"
    SSD_LITE_MOBILENET_V2 = "ssd_lite_mobilenet_v2"


class MapAccuracyMetricsToDLTask(str, Enum):
    TOP1 = DeepLearningTask.CLASSIFICATION.value
    TOP5 = DeepLearningTask.CLASSIFICATION.value
    AUC = DeepLearningTask.CLASSIFICATION.value
    PRECISION = DeepLearningTask.CLASSIFICATION.value
    RECALL = DeepLearningTask.CLASSIFICATION.value
    F1_SCORE = DeepLearningTask.CLASSIFICATION.value
    TRUE_POSITIVES = DeepLearningTask.CLASSIFICATION.value
    TRUE_NEGATIVES = DeepLearningTask.CLASSIFICATION.value
    FALSE_POSITIVES = DeepLearningTask.CLASSIFICATION.value
    FALSE_NEGATIVES = DeepLearningTask.CLASSIFICATION.value
    MIOU = DeepLearningTask.SEMANTIC_SEGMENTATION.value
    PIXEL_ACCURACY = DeepLearningTask.SEMANTIC_SEGMENTATION.value
    DICE_COEFFICIENT = DeepLearningTask.SEMANTIC_SEGMENTATION.value
    MAP = DeepLearningTask.OBJECT_DETECTION.value
    COCO_MAP = DeepLearningTask.OBJECT_DETECTION.value

    @staticmethod
    def get_accuracy_metrics_keys_by_dl_task(dl_task: "DeepLearningTask") -> "List[AccuracyMetricKey]":
        # use of __members__.items() is needed here because of the repetition of values in enum.
        return [AccuracyMetricKey[k] for k, v in MapAccuracyMetricsToDLTask.__members__.items() if v == dl_task]


class QuantizationLevel(str, Enum):
    FP32 = "FP32"
    FP16 = "FP16"
    INT8 = "INT8"
    HYBRID = "Hybrid"

    @staticmethod
    def from_string(quantization_level: str) -> "QuantizationLevel":
        quantization_level = quantization_level.lower()
        if quantization_level == "fp32":
            return QuantizationLevel.FP32
        elif quantization_level == "fp16":
            return QuantizationLevel.FP16
        elif quantization_level == "int8":
            return QuantizationLevel.INT8
        elif quantization_level == "hybrid":
            return QuantizationLevel.HYBRID
        else:
            raise NotImplementedError(f'Quantization Level: "{quantization_level}" is not supported')

    @property
    def numpy_type(self) -> "Type[Union[numpy.half, numpy.single, numpy.int8]]":
        map_quantization_level_to_numpy_type = {
            QuantizationLevel.FP32: numpy.float32,
            QuantizationLevel.FP16: numpy.float16,
            QuantizationLevel.INT8: numpy.int8,
        }
        return map_quantization_level_to_numpy_type[self]  # type: ignore[return-value]


def get_all_architectures_in_web_form() -> "List[str]":
    return get_all_enum_fields_in_web_form(Architecture, custom={"mmp": "MMP", "csp_darknet53": "CSPDarknet"})


def get_all_dl_tasks_in_web_form() -> "List[str]":
    """
    This function made to serve the platform ui.
    Returns:
        all dl tasks transformed from  'object_detection' to 'objectDetection'
    """
    return get_all_enum_fields_in_web_form(DeepLearningTask, custom={"nlp": "NLP"})



from enum import Enum

# TODO: rename HardwareType to HardwareName and inferenceHardware to HardwareType
from typing import Optional

from deci_common.data_types.enum.models_enums import BatchSize, BatchSizeEdge
from deci_common.helpers import get_enum_values

UNKNOWN = "UNKNOWN"


class HardwareType(str, Enum):
    """
    The type of the hardware to run on (CPU/GPU Names)
    """

    K80 = "K80"
    V100 = "V100"
    T4 = "T4"
    A10G = "A10G"
    A100_80G = "A100 80GB GCP"
    A100_40G = "A100 40GB GCP"
    EPYC = "EPYC"
    EPYC_7002 = "EPYC 7002"
    EPYC_7003 = "EPYC 7003"
    XAVIER = "Jetson Xavier"
    NANO = "Jetson Nano"
    XAVIER_AGX = "Jetson Xavier AGX"
    ORIN = "Jetson Orin"
    ORIN_NX = "Jetson Orin NX"
    ORIN_NANO = "Jetson Orin Nano"
    ORIN_NANO_4G = "Jetson Orin Nano 4GB"
    CASCADE_LAKE = "Cascade Lake"
    SKYLAKE = "Skylake"
    Broadwell = "Broadwell"
    Icelake = "Icelake"
    NUC_TIGER_LAKE = "Intel NUC Tiger Lake"
    SKYLAKE_SP = "Skylake-SP"
    CASCADE_LAKE_GCP = "Cascade Lake GCP"
    IMX8 = "NXP i.MX 8M mini"
    C5_2XLARGE = "c5.2xlarge"

    HAILO8 = "Hailo-8"
    AMBARELLA = "Ambarella"
    APPLE_IPHONE_A11 = "Apple iPhone A11"
    APPLE_IPHONE_A12 = "Apple iPhone A12"
    APPLE_IPHONE_A14 = "Apple iPhone A14"
    APPLE_IPHONE_A15 = "Apple iPhone A15"
    SNAPDRAGON_845 = "Snapdragon 845"
    SNAPDRAGON_855 = "Snapdragon 855"
    SNAPDRAGON_885 = "Snapdragon 888"
    SNAPDRAGON_8_GEN_1 = "Snapdragon 8 Gen 1"
    EXYNOS_9810 = "Exynos 9810"
    EXYNOS_9820 = "Exynos 9820"
    EXYNOS_990 = "Exynos 990"
    EXYNOS_2200 = "Exynos 2200"

    @property
    def is_future(self) -> bool:
        return self in [
            HardwareType.HAILO8,
            HardwareType.AMBARELLA,
            HardwareType.APPLE_IPHONE_A11,
            HardwareType.APPLE_IPHONE_A12,
            HardwareType.APPLE_IPHONE_A14,
            HardwareType.APPLE_IPHONE_A15,
            HardwareType.SNAPDRAGON_845,
            HardwareType.SNAPDRAGON_855,
            HardwareType.SNAPDRAGON_885,
            HardwareType.SNAPDRAGON_8_GEN_1,
            HardwareType.EXYNOS_9810,
            HardwareType.EXYNOS_9820,
            HardwareType.EXYNOS_990,
            HardwareType.EXYNOS_2200,
        ]

    @property
    def is_deprecated(self) -> bool:
        return self in [HardwareType.K80, HardwareType.EPYC]


class HardwareGroup(str, Enum):
    CPU = "CPU"
    GPU = "GPU"
    COMMERCIAL_EDGE = "Commercial Edge"
    CONSUMER_EDGE = "Consumer Edge"


class HardwareMachineModel(str, Enum):
    K80 = "p2.xlarge"
    V100 = "p3.2xlarge"
    T4 = "g4dn.2xlarge"
    A10G = "g5.2xlarge"
    EPYC = "c5a.2xlarge"
    CASCADE_LAKE = "c5.4xlarge"
    SKYLAKE = "c5n.4xlarge"
    Broadwell = "m4.4xlarge"
    Icelake = "m6i.4xlarge"
    EPYC_7002 = "c5a.4xlarge"
    EPYC_7003 = "m6a.4xlarge"
    SKYLAKE_SP = "m5.2xlarge"
    A100_40G = "a2-highgpu-1g"
    CASCADE_LAKE_GCP = "n2-standard-4"
    C5_2XLARGE = "c5.2xlarge"

    NANO = HardwareType.NANO.value
    XAVIER = HardwareType.XAVIER.value
    XAVIER_AGX = HardwareType.XAVIER_AGX.value
    ORIN = HardwareType.ORIN.value
    ORIN_NX = HardwareType.ORIN_NX.value
    ORIN_NANO = HardwareType.ORIN_NANO.value
    ORIN_NANO_4G = HardwareType.ORIN_NANO_4G.value
    NUC_TIGER_LAKE = HardwareType.NUC_TIGER_LAKE.value
    A100_80G = HardwareType.A100_80G.value
    IMX8 = HardwareType.IMX8.value
    HAILO8 = HardwareType.HAILO8.value
    AMBARELLA = HardwareType.AMBARELLA.value
    APPLE_IPHONE_A11 = HardwareType.APPLE_IPHONE_A11.value
    APPLE_IPHONE_A12 = HardwareType.APPLE_IPHONE_A12.value
    APPLE_IPHONE_A14 = HardwareType.APPLE_IPHONE_A14.value
    APPLE_IPHONE_A15 = HardwareType.APPLE_IPHONE_A15.value
    SNAPDRAGON_845 = HardwareType.SNAPDRAGON_845.value
    SNAPDRAGON_855 = HardwareType.SNAPDRAGON_855.value
    SNAPDRAGON_885 = HardwareType.SNAPDRAGON_885.value
    SNAPDRAGON_8_GEN_1 = HardwareType.SNAPDRAGON_8_GEN_1.value
    EXYNOS_9810 = HardwareType.EXYNOS_9810.value
    EXYNOS_9820 = HardwareType.EXYNOS_9820.value
    EXYNOS_990 = HardwareType.EXYNOS_990.value
    EXYNOS_2200 = HardwareType.EXYNOS_2200.value


class InferenceHardware(str, Enum):
    """
    Hardware that can be used for deep learning inference.
    """

    CPU = "cpu"
    GPU = "gpu"


class MapHardwareTypeToFamily(str, Enum):
    # GPUs
    V100 = InferenceHardware.GPU.value
    K80 = InferenceHardware.GPU.value
    T4 = InferenceHardware.GPU.value
    A10G = InferenceHardware.GPU.value
    A100_80G = InferenceHardware.GPU.value
    A100_40G = InferenceHardware.GPU.value
    NANO = InferenceHardware.GPU.value
    XAVIER = InferenceHardware.GPU.value
    XAVIER_AGX = InferenceHardware.GPU.value
    ORIN = InferenceHardware.GPU.value
    ORIN_NX = InferenceHardware.GPU.value
    ORIN_NANO = InferenceHardware.GPU.value
    ORIN_NANO_4G = InferenceHardware.GPU.value

    # CPUs
    EPYC = InferenceHardware.CPU.value
    EPYC_7002 = InferenceHardware.CPU.value
    EPYC_7003 = InferenceHardware.CPU.value
    CASCADE_LAKE = InferenceHardware.CPU.value
    SKYLAKE = InferenceHardware.CPU.value
    Broadwell = InferenceHardware.CPU.value
    Icelake = InferenceHardware.CPU.value
    NUC_TIGER_LAKE = InferenceHardware.CPU.value
    SKYLAKE_SP = InferenceHardware.CPU.value
    CASCADE_LAKE_GCP = InferenceHardware.CPU.value
    IMX8 = InferenceHardware.CPU.value
    C5_2XLARGE = InferenceHardware.CPU.value


class InferyVersion(str, Enum):
    CPU = "cpu"
    GPU = "gpu"
    JETSON = "jetson"
    JETSON_PY36 = "jetson_py36"
    JETSON_PY38 = "jetson_py38"


class MapHardwareTypeToInferyVersion(str, Enum):
    # GPUs
    V100 = InferyVersion.GPU.value
    K80 = InferyVersion.GPU.value
    T4 = InferyVersion.GPU.value
    A10G = InferyVersion.GPU.value
    A100_80G = InferyVersion.GPU.value
    A100_40G = InferyVersion.GPU.value

    # Jetsons
    NANO = InferyVersion.JETSON_PY36.value
    XAVIER = InferyVersion.JETSON_PY38.value
    XAVIER_AGX = InferyVersion.JETSON_PY36.value
    ORIN = InferyVersion.JETSON_PY38.value
    ORIN_NX = InferyVersion.JETSON_PY38.value
    ORIN_NANO = InferyVersion.JETSON_PY38.value
    ORIN_NANO_4G = InferyVersion.JETSON_PY38.value

    # CPUs
    EPYC = InferyVersion.CPU.value
    EPYC_7002 = InferyVersion.CPU.value
    EPYC_7003 = InferyVersion.CPU.value
    CASCADE_LAKE = InferyVersion.CPU.value
    SKYLAKE = InferyVersion.CPU.value
    Broadwell = InferyVersion.CPU.value
    Icelake = InferyVersion.CPU.value
    NUC_TIGER_LAKE = InferyVersion.CPU.value
    SKYLAKE_SP = InferyVersion.CPU.value
    CASCADE_LAKE_GCP = InferyVersion.CPU.value
    IMX8 = InferyVersion.CPU.value
    C5_2XLARGE = InferyVersion.CPU.value


class HardwareEnvironment(str, Enum):
    GCP = "gcp"
    AWS = "aws"
    Azure = "azure"
    PREMISE = "on premise"


class HardwareVendor(str, Enum):
    INTEL = "intel"
    NVIDIA = "nvidia"
    AMD = "amd"
    NXP = "nxp"


class HardwareTaint(str, Enum):
    # GPUs
    V100 = "nvidia.com/v100gpu"
    K80 = "nvidia.com/k80gpu"
    T4 = "nvidia.com/t4gpu"
    A10G = "nvidia.com/a10gpu"
    A100_80G = UNKNOWN
    A100_40G = "nvidia.com/a100gpu40g"
    NANO = "nvidia.com/jetson-nano"
    XAVIER = "nvidia.com/jetson-xavier"
    XAVIER_AGX = "nvidia.com/jetson-xavier-agx"
    ORIN = "nvidia.com/jetson-orin"
    ORIN_NX = "nvidia.com/jetson-orin-as-nx16"
    ORIN_NANO = "nvidia.com/jetson-orin-as-nano"
    ORIN_NANO_4G = "nvidia.com/jetson-orin-as-nano4g"
    # CPUs
    EPYC = "amd.com/epyc"
    EPYC_7002 = "amd.com/7002"
    EPYC_7003 = "amd.com/7003"
    CASCADE_LAKE = "intel.com/cascade-lake"
    SKYLAKE = "intel.com/skylake"
    Broadwell = "intel.com/broadwell"
    Icelake = "intel.com/icelake"
    NUC_TIGER_LAKE = "intel.com/nuc-tiger-lake"
    SKYLAKE_SP = "intel.com/skylake-sp"
    CASCADE_LAKE_GCP = "intel.com/xeon"
    IMX8 = UNKNOWN
    C5_2XLARGE = "intel.com/c5-2xlarge"


class HardwareLabel(str, Enum):
    # GPUs
    V100 = "nvidia-tesla-v100"
    K80 = "nvidia-tesla-k80"
    T4 = "nvidia-tesla-t4"
    A10G = "nvidia-ampere-a10"
    A100_80G = UNKNOWN
    A100_40G = "nvidia-a100-40g"
    NANO = "nvidia-jetson-nano"
    XAVIER = "nvidia-jetson-xavier"
    XAVIER_AGX = "nvidia-jetson-xavier-agx"
    ORIN = "nvidia-jetson-orin"
    ORIN_NX = "nvidia-jetson-orin-as-nx16"
    ORIN_NANO = "nvidia-jetson-orin-as-nano"
    ORIN_NANO_4G = "nvidia-jetson-orin-as-nano4g"

    # CPUs
    EPYC = "amd-epyc"
    EPYC_7002 = "amd-epyc-7002"
    EPYC_7003 = "amd-epyc-7003"
    CASCADE_LAKE = "intel-cascade-lake"
    SKYLAKE = "intel-skylake"
    Broadwell = "intel-broadwell"
    Icelake = "intel-icelake"
    NUC_TIGER_LAKE = "intel-nuc-tiger-lake"
    SKYLAKE_SP = "intel-skylake-sp"
    CASCADE_LAKE_GCP = "intel-xeon"
    IMX8 = UNKNOWN
    C5_2XLARGE = "intel-c5-2xlarge"


class MapHardwareTypeToVendor(str, Enum):
    V100 = HardwareVendor.NVIDIA.value
    K80 = HardwareVendor.NVIDIA.value
    T4 = HardwareVendor.NVIDIA.value
    A10G = HardwareVendor.NVIDIA.value
    A100_80G = HardwareVendor.NVIDIA.value
    A100_40G = HardwareVendor.NVIDIA.value
    EPYC = HardwareVendor.AMD.value
    EPYC_7002 = HardwareVendor.AMD.value
    EPYC_7003 = HardwareVendor.AMD.value
    NANO = HardwareVendor.NVIDIA.value
    XAVIER = HardwareVendor.NVIDIA.value
    XAVIER_AGX = HardwareVendor.NVIDIA.value
    ORIN = HardwareVendor.NVIDIA.value
    ORIN_NX = HardwareVendor.NVIDIA.value
    ORIN_NANO = HardwareVendor.NVIDIA.value
    ORIN_NANO_4G = HardwareVendor.NVIDIA.value
    CASCADE_LAKE = HardwareVendor.INTEL.value
    SKYLAKE = HardwareVendor.INTEL.value
    Broadwell = HardwareVendor.INTEL.value
    Icelake = HardwareVendor.INTEL.value
    NUC_TIGER_LAKE = HardwareVendor.INTEL.value
    SKYLAKE_SP = HardwareVendor.INTEL.value
    CASCADE_LAKE_GCP = HardwareVendor.INTEL.value
    C5_2XLARGE = HardwareVendor.INTEL.value
    IMX8 = HardwareVendor.NXP.value


class MapHardwareTypeToEnvironment(str, Enum):
    # AWS
    V100 = HardwareEnvironment.AWS.value
    K80 = HardwareEnvironment.AWS.value
    T4 = HardwareEnvironment.AWS.value
    A10G = HardwareEnvironment.AWS.value
    EPYC = HardwareEnvironment.AWS.value
    EPYC_7002 = HardwareEnvironment.AWS.value
    EPYC_7003 = HardwareEnvironment.AWS.value
    CASCADE_LAKE = HardwareEnvironment.AWS.value
    SKYLAKE = HardwareEnvironment.AWS.value
    Broadwell = HardwareEnvironment.AWS.value
    Icelake = HardwareEnvironment.AWS.value
    SKYLAKE_SP = HardwareEnvironment.AWS.value
    C5_2XLARGE = HardwareEnvironment.AWS.value

    # GCP
    A100_40G = HardwareEnvironment.GCP.value
    A100_80G = HardwareEnvironment.GCP.value
    CASCADE_LAKE_GCP = HardwareEnvironment.GCP.value

    # PREMISE
    NANO = HardwareEnvironment.PREMISE.value
    XAVIER = HardwareEnvironment.PREMISE.value
    XAVIER_AGX = HardwareEnvironment.PREMISE.value
    ORIN = HardwareEnvironment.PREMISE.value
    ORIN_NX = HardwareEnvironment.PREMISE.value
    ORIN_NANO = HardwareEnvironment.PREMISE.value
    ORIN_NANO_4G = HardwareEnvironment.PREMISE.value
    NUC_TIGER_LAKE = HardwareEnvironment.PREMISE.value
    IMX8 = HardwareEnvironment.PREMISE.value


class MapHardwareTypeToDefaultBatchSizeList(list, Enum):  # type: ignore[type-arg]
    V100 = get_enum_values(BatchSize)
    K80 = get_enum_values(BatchSize)
    T4 = get_enum_values(BatchSize)
    A10G = get_enum_values(BatchSize)
    A100_80G = get_enum_values(BatchSize)
    A100_40G = get_enum_values(BatchSize)
    EPYC = get_enum_values(BatchSize)
    EPYC_7002 = get_enum_values(BatchSize)
    EPYC_7003 = get_enum_values(BatchSize)
    NANO = get_enum_values(BatchSizeEdge)
    XAVIER = get_enum_values(BatchSizeEdge)
    XAVIER_AGX = get_enum_values(BatchSizeEdge)
    ORIN = get_enum_values(BatchSizeEdge)
    ORIN_NX = get_enum_values(BatchSizeEdge)
    ORIN_NANO = get_enum_values(BatchSizeEdge)
    ORIN_NANO_4G = get_enum_values(BatchSizeEdge)
    CASCADE_LAKE = get_enum_values(BatchSize)
    SKYLAKE = get_enum_values(BatchSize)
    Broadwell = get_enum_values(BatchSize)
    Icelake = get_enum_values(BatchSize)
    NUC_TIGER_LAKE = get_enum_values(BatchSize)
    SKYLAKE_SP = get_enum_values(BatchSize)
    CASCADE_LAKE_GCP = get_enum_values(BatchSize)
    IMX8 = get_enum_values(BatchSize)
    C5_2XLARGE = get_enum_values(BatchSize)


class HardwareImageRepository(str, Enum):
    INTEL = "intel"
    JETSON = "jetson"


class MapHardwareTypeToImageRepository(str, Enum):
    NANO = HardwareImageRepository.JETSON.value
    XAVIER = HardwareImageRepository.JETSON.value
    XAVIER_AGX = HardwareImageRepository.JETSON.value
    ORIN = HardwareImageRepository.JETSON.value
    ORIN_NX = HardwareImageRepository.JETSON.value
    ORIN_NANO = HardwareImageRepository.JETSON.value
    ORIN_NANO_4G = HardwareImageRepository.JETSON.value
    NUC_TIGER_LAKE = HardwareImageRepository.INTEL.value


class HardwareImageDistribution(str, Enum):
    J46 = "j46"
    J50 = "j50"
    J502 = "j502"

    @property
    def python_version(self) -> str:
        dist_to_version = {
            HardwareImageDistribution.J46: "3.6",
            HardwareImageDistribution.J50: "3.8",
            HardwareImageDistribution.J502: "3.8",
        }
        return dist_to_version[self]


class MapHardwareTypeToImageDistribution(str, Enum):
    NANO = HardwareImageDistribution.J46.value
    XAVIER = HardwareImageDistribution.J502.value
    XAVIER_AGX = HardwareImageDistribution.J46.value
    ORIN = HardwareImageDistribution.J502.value
    ORIN_NX = HardwareImageDistribution.J502.value
    ORIN_NANO = HardwareImageDistribution.J502.value
    ORIN_NANO_4G = HardwareImageDistribution.J502.value


class MapHardwareTypeToGroup(str, Enum):
    K80 = HardwareGroup.GPU.value
    V100 = HardwareGroup.GPU.value
    T4 = HardwareGroup.GPU.value
    A10G = HardwareGroup.GPU.value
    A100_80G = HardwareGroup.GPU.value
    A100_40G = HardwareGroup.GPU.value

    XAVIER = HardwareGroup.COMMERCIAL_EDGE.value
    NANO = HardwareGroup.COMMERCIAL_EDGE.value
    XAVIER_AGX = HardwareGroup.COMMERCIAL_EDGE.value
    ORIN = HardwareGroup.COMMERCIAL_EDGE.value
    ORIN_NX = HardwareGroup.COMMERCIAL_EDGE.value
    ORIN_NANO = HardwareGroup.COMMERCIAL_EDGE.value
    ORIN_NANO_4G = HardwareGroup.COMMERCIAL_EDGE.value
    NUC_TIGER_LAKE = HardwareGroup.COMMERCIAL_EDGE.value
    AMBARELLA = HardwareGroup.COMMERCIAL_EDGE.value
    HAILO8 = HardwareGroup.COMMERCIAL_EDGE.value
    IMX8 = HardwareGroup.COMMERCIAL_EDGE.value

    EPYC = HardwareGroup.CPU.value
    EPYC_7002 = HardwareGroup.CPU.value
    EPYC_7003 = HardwareGroup.CPU.value
    CASCADE_LAKE = HardwareGroup.CPU.value
    SKYLAKE = HardwareGroup.CPU.value
    Broadwell = HardwareGroup.CPU.value
    Icelake = HardwareGroup.CPU.value
    SKYLAKE_SP = HardwareGroup.CPU.value
    CASCADE_LAKE_GCP = HardwareGroup.CPU.value
    C5_2XLARGE = HardwareGroup.CPU.value

    APPLE_IPHONE_A11 = HardwareGroup.CONSUMER_EDGE.value
    APPLE_IPHONE_A12 = HardwareGroup.CONSUMER_EDGE.value
    APPLE_IPHONE_A14 = HardwareGroup.CONSUMER_EDGE.value
    APPLE_IPHONE_A15 = HardwareGroup.CONSUMER_EDGE.value
    SNAPDRAGON_845 = HardwareGroup.CONSUMER_EDGE.value
    SNAPDRAGON_855 = HardwareGroup.CONSUMER_EDGE.value
    SNAPDRAGON_885 = HardwareGroup.CONSUMER_EDGE.value
    SNAPDRAGON_8_GEN_1 = HardwareGroup.CONSUMER_EDGE.value
    EXYNOS_9810 = HardwareGroup.CONSUMER_EDGE.value
    EXYNOS_9820 = HardwareGroup.CONSUMER_EDGE.value
    EXYNOS_990 = HardwareGroup.CONSUMER_EDGE.value
    EXYNOS_2200 = HardwareGroup.CONSUMER_EDGE.value


def type_with_model(model: HardwareMachineModel, override_label: Optional[str] = None) -> str:
    return f"{override_label or HardwareType[model.name].value} ({model})"


class HardwareTypeLabel(str, Enum):
    XAVIER = "Jetson Xavier NX 16GB"
    NANO = "Jetson Nano 4GB"
    XAVIER_AGX = "Jetson AGX Xavier 32GB"
    ORIN = "Jetson AGX Orin Development Kit"
    ORIN_NX = "Jetson Orin NX 16GB"
    ORIN_NANO = "Jetson Orin Nano 8GB"
    ORIN_NANO_4G = "Jetson Orin Nano 4GB"
    EPYC = type_with_model(HardwareMachineModel.EPYC)
    EPYC_7002 = type_with_model(HardwareMachineModel.EPYC_7002, f"AMD Rome {HardwareType.EPYC_7002.value}")
    EPYC_7003 = type_with_model(HardwareMachineModel.EPYC_7003, f"AMD Milan {HardwareType.EPYC_7003.value}")
    CASCADE_LAKE = type_with_model(HardwareMachineModel.CASCADE_LAKE)
    SKYLAKE = type_with_model(HardwareMachineModel.SKYLAKE, "Sky Lake")
    Broadwell = type_with_model(HardwareMachineModel.Broadwell)
    Icelake = type_with_model(HardwareMachineModel.Icelake, "Ice Lake")
    K80 = type_with_model(HardwareMachineModel.K80)
    V100 = type_with_model(HardwareMachineModel.V100)
    SKYLAKE_SP = type_with_model(HardwareMachineModel.SKYLAKE_SP)
    T4 = type_with_model(HardwareMachineModel.T4)
    A10G = type_with_model(HardwareMachineModel.A10G)
    A100_40G = type_with_model(HardwareMachineModel.A100_40G, "A100 40GB")
    CASCADE_LAKE_GCP = type_with_model(HardwareMachineModel.CASCADE_LAKE_GCP, "Cascade Lake")

    A100_80G = HardwareType.A100_80G.value
    NUC_TIGER_LAKE = HardwareType.NUC_TIGER_LAKE.value
    IMX8 = HardwareType.IMX8.value
    C5_2XLARGE = HardwareType.C5_2XLARGE.value
    HAILO8 = HardwareType.HAILO8.value
    AMBARELLA = HardwareType.AMBARELLA.value
    APPLE_IPHONE_A11 = HardwareType.APPLE_IPHONE_A11.value
    APPLE_IPHONE_A12 = HardwareType.APPLE_IPHONE_A12.value
    APPLE_IPHONE_A14 = HardwareType.APPLE_IPHONE_A14.value
    APPLE_IPHONE_A15 = HardwareType.APPLE_IPHONE_A15.value
    SNAPDRAGON_845 = HardwareType.SNAPDRAGON_845.value
    SNAPDRAGON_855 = HardwareType.SNAPDRAGON_855.value
    SNAPDRAGON_885 = HardwareType.SNAPDRAGON_885.value
    SNAPDRAGON_8_GEN_1 = HardwareType.SNAPDRAGON_8_GEN_1.value
    EXYNOS_9810 = HardwareType.EXYNOS_9810.value
    EXYNOS_9820 = HardwareType.EXYNOS_9820.value
    EXYNOS_990 = HardwareType.EXYNOS_990.value
    EXYNOS_2200 = HardwareType.EXYNOS_2200.value



from abc import ABC
from typing import TYPE_CHECKING, Any, Dict

from pymongo import MongoClient

from deci_common.abstractions.abstract_logger import ILogger

if TYPE_CHECKING:
    from typing import Optional, Type

    from pymongo.database import Database


class MongoConfiguration(ABC):
    DB_NAME: str
    HOST: str
    PORT: int
    USERNAME: str
    PASSWORD: str
    AUTHENTICATION_DB_NAME: str  # The database on which to preform the authentication


class MongoConnector(ILogger):
    """
    An abstract MongoDBConnector that operates on a collection (SQL Table equivalent).
    """

    def __init__(self, mongo_db_configuration: "Type[MongoConfiguration]"):
        self._db_config: "Type[MongoConfiguration]" = mongo_db_configuration
        self._client: "Optional[MongoClient[Dict[str, Any]]]" = None
        super().__init__()

    @property
    def db(self) -> "Database[Dict[str, Any]]":
        if self._client is None:
            self._client = self.__create_client()
        return self._client[self._db_config.DB_NAME]

    def __create_client(self) -> "MongoClient[Dict[str, Any]]":
        """
        Creates a new pymongo.MongoClient for DB operations.
        """
        client = MongoClient[Dict[str, Any]](
            host=self._db_config.HOST,
            port=self._db_config.PORT,
            authSource=self._db_config.AUTHENTICATION_DB_NAME,
            username=self._db_config.USERNAME,
            password=self._db_config.PASSWORD,
            uuidRepresentation="pythonLegacy",
            retryWrites=False,
            connect=True,
            tls=True,
        )
        self._logger.debug("Authenticated to db successfully.")

        return client

    def ping(self) -> "Dict[str, Any]":
        return self.db.command("ping")


__all__ = ["MongoConfiguration", "MongoConnector"]



from datetime import datetime
from typing import TYPE_CHECKING
from warnings import warn

from deci_common.abstractions.abstract_logger import ILogger

if TYPE_CHECKING:
    from typing import Any, Dict, List, Optional

    from pymongo.collection import Collection
    from pymongo.command_cursor import CommandCursor
    from pymongo.cursor import Cursor
    from pymongo.results import DeleteResult, InsertOneResult, UpdateResult

    from deci_common.data_connection.mongo_connector import MongoConnector


class MongoDBInterface(ILogger):
    # FIXME: https://deci.atlassian.net/browse/PLFM-3408
    _CREATION_TIME = "creation_time"
    _UPDATE_TIME = "update_time"

    def __init__(self, collection: str, mongo_connector: "MongoConnector"):
        super().__init__()
        self._collection = collection
        self._mongo_connector = mongo_connector

    @property
    def collection(self) -> "Collection[Dict[str, Any]]":
        return self._mongo_connector.db[self._collection]

    @property
    def _update_time_partial(self) -> "Dict[str, datetime]":
        return {self._UPDATE_TIME: datetime.utcnow()}

    @property
    def _creation_time_partial(self) -> "Dict[str, datetime]":
        return {self._CREATION_TIME: datetime.utcnow()}

    def delete(self, search_filter: "Dict[str, Any]") -> "DeleteResult":
        """
        Delete documents from the DB that match the passed search_filter.
        @param search_filter: The document to insert
        @return: DeleteResult
        """
        return self.collection.delete_many(search_filter)

    def insert(self, document: "Dict[str, Any]") -> "InsertOneResult":
        """
        Insert a document to the DB.
        @param document: The document to insert
        @return: InsertOneResult
        """
        document.update({**self._creation_time_partial, **self._update_time_partial})
        return self.collection.insert_one(document=document)

    def find_one(
        self,
        search_filter: "Dict[str, Any]",
        *,
        projection: "Dict[str, Any]" = {},
    ) -> "Optional[Dict[str, Any]]":
        """
        Find single element from DB with a specified search filter
        @param search_filter: Filter for the search
        @param projection: Projection to apply
        @return: Dict[str, Any]
        """
        return self.collection.find_one(filter=search_filter, projection=projection)

    def find_by_id(self, _id: "Any", *, id_key: str = "id") -> "Optional[Dict[str, Any]]":
        """
        Find single element from DB by ID
        @param _id: ID value
        @param id_key: Name of field containing ID in DB. Optional, defaults to "id".
        @return: Dict[str, Any]
        """
        return self.find_one(search_filter={id_key: _id})

    def find(
        self,
        search_filter: "Dict[str, Any]" = {},
        *,
        projection: "Dict[str, Any]" = {},
    ) -> "Cursor[Dict[str, Any]]":
        """
        Find all elements from DB with a specified search filter
        @param search_filter: Filter for the search
        @param projection: Projection to apply
        @return: Cursor[Dict[str, Any]
        """
        return self.collection.find(filter=search_filter, projection=projection)

    def _update(
        self,
        *,
        search_filter: "Dict[str, Any]",
        update: "Dict[str, Any]",
        upsert: bool,
    ) -> "UpdateResult":
        """
        Update all elements from DB that match a specified search filter with the updated document
        @param search_filter: Filter for the search
        @param update: Update to apply
        @return: UpdateResult
        """
        return self.collection.update_many(filter=search_filter, update=update, upsert=upsert)

    def update(
        self,
        search_filter: "Dict[str, Any]",
        *,
        updated_document: "Dict[str, Any]",
        upsert: bool = False,
    ) -> "UpdateResult":
        if upsert:
            warn("Passing `upsert=True` is deprecated, please fix usage.")
        updated_document.update(self._update_time_partial)
        return self._update(search_filter=search_filter, update={"$set": updated_document}, upsert=upsert)

    def upsert(self, updated_document: "Dict[str, Any]", *, id_key: str = "id") -> "Dict[str, Any]":
        search_filter = {id_key: updated_document[id_key]}
        updated_document.update(self._update_time_partial)
        updated = self._update(search_filter=search_filter, update={"$set": updated_document}, upsert=False)
        if updated.modified_count == 0:
            self.insert(document=updated_document)
        return self.find_by_id(search_filter[id_key], id_key=id_key)  # type: ignore[return-value]

    def remove_from_list(self, search_filter: "Dict[str, Any]", *, remove_query: "Dict[str, Any]") -> "UpdateResult":
        return self._update(
            search_filter=search_filter,
            update={"$pull": remove_query, "$set": self._update_time_partial},
            upsert=False,
        )

    def add_to_list(self, search_filter: "Dict[str, Any]", *, add_query: "Dict[str, Any]") -> "UpdateResult":
        return self._update(
            search_filter=search_filter,
            update={"$addToSet": add_query, "$set": self._update_time_partial},
            upsert=False,
        )

    def increment_field(
        self,
        search_filter: "Dict[str, Any]",
        *,
        increment_query: "Dict[str, Any]",
        upsert: bool = False,
    ) -> bool:
        update_result = self._update(
            search_filter=search_filter,
            update={"$inc": increment_query, "$set": self._update_time_partial},
            upsert=upsert,
        )
        return update_result.modified_count != 0

    def aggregate(self, pipeline: "List[Dict[str, Any]]") -> "CommandCursor[Dict[str, Any]]":
        return self.collection.aggregate(pipeline=pipeline)

    def ping(self) -> "Dict[str, Any]":
        return self._mongo_connector.ping()



import json
from typing import Optional

from deci_common.abstractions.base_model import Schema


class InferencerBenchmarkResult(Schema):
    batch_size: Optional[int] = None
    batch_inf_time: Optional[float] = None
    batch_inf_time_variance: Optional[float] = None

    memory: Optional[float] = None

    pre_inference_memory_used: Optional[float] = None
    post_inference_memory_used: Optional[float] = None
    total_memory_size: Optional[float] = None
    throughput: Optional[float] = None
    sample_inf_time: Optional[float] = None
    include_io: Optional[bool] = None
    framework_type: Optional[str] = None
    framework_version: Optional[str] = None
    inference_hardware: Optional[str] = None
    infery_version: Optional[str] = None
    date: Optional[str] = None
    ctime: Optional[int] = None
    h_to_d_mean: Optional[float] = None
    d_to_h_mean: Optional[float] = None
    h_to_d_variance: Optional[float] = None
    d_to_h_variance: Optional[float] = None

    def __str__(self) -> str:
        benchmarks_dict = self.dict()

        # Adding ms and fps to __repr__ output.
        results_benchmarks_dict = dict()
        for k, v in benchmarks_dict.items():
            if isinstance(v, float):
                if k == "throughput":
                    results_benchmarks_dict[k] = f"{v:.2f} fps"
                elif "memory" in k:
                    results_benchmarks_dict[k] = f"{v:.2f} mb"
                else:
                    results_benchmarks_dict[k] = f"{v:.2f} ms"
            else:
                results_benchmarks_dict[k] = v

        return f"<ModelBenchmarks: {json.dumps(results_benchmarks_dict, indent=4)}>"

    def __repr__(self) -> str:
        return str(self)



from typing import Optional

from deci_common.abstractions.base_model import Schema
from deci_common.data_types.enum.hardware_enums import (
    HardwareEnvironment,
    HardwareGroup,
    HardwareImageDistribution,
    HardwareImageRepository,
    HardwareLabel,
    HardwareMachineModel,
    HardwareTaint,
    HardwareType,
    HardwareTypeLabel,
    HardwareVendor,
    InferenceHardware,
    MapHardwareTypeToEnvironment,
    MapHardwareTypeToFamily,
    MapHardwareTypeToGroup,
    MapHardwareTypeToImageDistribution,
    MapHardwareTypeToImageRepository,
    MapHardwareTypeToVendor,
)


# TODO move all hardwaretype usage to use thic class insted.
class HardwareReturnSchema(Schema):
    """
    A logic schema of hardware
    """

    name: HardwareType
    label: HardwareTypeLabel
    vendor: Optional[HardwareVendor] = None
    machine: Optional[HardwareMachineModel] = None
    group: Optional[HardwareGroup] = None
    future: bool = False
    deprecated: bool = False


class Hardware(Schema):
    """
    A logic schema of hardware
    """

    name: HardwareType
    type: InferenceHardware
    # equals None for backwards compatibility of benchmark jobs who does not have it.
    machine: Optional[HardwareMachineModel] = None
    cost_per_hour: int = 0
    environment: Optional[HardwareEnvironment]
    vendor: HardwareVendor
    label: HardwareLabel
    taint: HardwareTaint
    image_repository: Optional[HardwareImageRepository]
    image_distribution: Optional[HardwareImageDistribution]


def get_hardware_by_hardware_name(hw_name: HardwareType) -> Hardware:
    if type(hw_name) is str:
        hw_name = HardwareType(hw_name)
    inference_hardware = MapHardwareTypeToFamily[hw_name.name].value
    image_repository = getattr(MapHardwareTypeToImageRepository, hw_name.name, None)
    image_distribution = getattr(MapHardwareTypeToImageDistribution, hw_name.name, None)
    hardware_environment = getattr(MapHardwareTypeToEnvironment, hw_name.name, None)
    return Hardware(
        name=hw_name,
        type=inference_hardware,  # type: ignore[arg-type]
        environment=hardware_environment.value if hardware_environment is not None else None,
        taint=HardwareTaint[hw_name.name],
        machine=HardwareMachineModel[hw_name.name],
        vendor=HardwareVendor.INTEL if inference_hardware == InferenceHardware.CPU else HardwareVendor.NVIDIA,
        label=HardwareLabel[hw_name.name],
        image_repository=image_repository.value if image_repository is not None else None,
        image_distribution=image_distribution.value if image_distribution is not None else None,
    )


def get_hardware_return_schema(hw_name: HardwareType) -> HardwareReturnSchema:
    if type(hw_name) is str:
        hw_name = HardwareType(hw_name)
    return HardwareReturnSchema(
        name=hw_name,
        label=HardwareTypeLabel[hw_name.name],
        vendor=MapHardwareTypeToVendor[hw_name.name].value if hasattr(MapHardwareTypeToVendor, hw_name.name) else None,  # type: ignore[arg-type]
        machine=HardwareMachineModel[hw_name.name],
        group=MapHardwareTypeToGroup[hw_name.name].value,  # type: ignore[arg-type]
        future=hw_name.is_future,
        deprecated=hw_name.is_deprecated,
    )



from enum import Enum


class OpenVinoCheckpointTypes(str, Enum):
    """
    When passing open vino path to checkpoint, it can be in variant types warping the base xml and bin files.

    Attributes:
    ZIP = zip file contains the xml and bin.
    PICKLE =  pkl file wrapping the xml and bin.
    ORIGIN = xml and bin file.
    """

    ZIP = "zip"
    PICKLE = "pkl"
    XML = "xml"



from enum import Enum
from typing import List, Optional

from deci_common.abstractions.base_model import Schema
from deci_common.data_types.enum.hardware_enums import InferenceHardware


class FileExtensions(Schema):
    main: str
    additional: List[str] = []
    raw: Optional[str] = None


class FrameworkFileExtension(Enum):
    TENSORFLOW1 = FileExtensions(main=".pb")
    TENSORFLOW2 = FileExtensions(main=".zip")
    PYTORCH = FileExtensions(main=".pth", additional=[".pt"])
    ONNX = FileExtensions(main=".onnx")
    TENSORRT = FileExtensions(main=".pkl", raw=".engine")
    OPENVINO = FileExtensions(main=".pkl", raw=".zip")
    TORCHSCRIPT = FileExtensions(main=".pth", additional=[".pt"])
    TVM = None
    KERAS = FileExtensions(main=".h5")
    TFLITE = FileExtensions(main=".tflite")
    COREML = FileExtensions(main=".mlmodel")
    TFJS = FileExtensions(main=".zip")


class FrameworkType(str, Enum):
    """
    A general deep learning framework, without a version.
    """

    TENSORFLOW1 = "tf1"
    TENSORFLOW2 = "tf2"
    PYTORCH = "pytorch"
    ONNX = "onnx"
    TORCHSCRIPT = "torchscript"
    TENSORRT = "trt"
    TVM = "tvm"
    OPENVINO = "openvino"
    KERAS = "keras"
    TFLITE = "tflite"
    COREML = "coreml"
    TFJS = "tfjs"

    @staticmethod
    def from_string(framework: str) -> Enum:
        framework = framework.lower()
        return FrameworkType(framework)


def get_hardware_families_by_framework_type(framework: FrameworkType) -> List[InferenceHardware]:
    if framework == FrameworkType.OPENVINO:
        return [InferenceHardware.CPU]
    elif framework == FrameworkType.TENSORRT:
        return [InferenceHardware.GPU]
    else:
        return [InferenceHardware.GPU, InferenceHardware.CPU]



from datetime import datetime
from typing import Optional
from uuid import UUID, uuid4

from pydantic import BaseModel, Field
from stringcase import camelcase  # type: ignore[import]


def to_camel(string: str) -> str:
    return camelcase(string)


class Schema(BaseModel):
    """
    A base class for all of Deci's model classes.
    A model stores data in constant fields, and let us manipulate the data in a more readable way.
    """

    class Config:
        alias_generator = to_camel
        allow_population_by_field_name = True
        validate_assignment = True
        allow_inf_nan = False

    pass


class DBSchema(Schema):
    update_time: Optional[datetime] = None
    creation_time: Optional[datetime] = None
    id: UUID = Field(default_factory=uuid4)
    deleted: bool = False



from typing import TYPE_CHECKING, Generic, TypeVar

from deci_common.abstractions.abstract_logger import ILogger
from deci_common.abstractions.base_model import DBSchema

if TYPE_CHECKING:
    from typing import Any, Dict, List, Optional, Type
    from uuid import UUID

    from deci_common.data_interfaces.mongo_db_interface import MongoDBInterface

T = TypeVar("T", bound=DBSchema)


class DBInterface(Generic[T], ILogger):
    def __init__(self, element_type: "Type[T]", mongo_db_interface: "MongoDBInterface"):
        super().__init__()
        self._element_type = element_type
        self._mongo_db_interface = mongo_db_interface

    def insert(self, element: "T") -> "T":
        """
        Insert an element to the DB and return it
        @param element: The element to insert
        @return: T
        """
        self._mongo_db_interface.insert(element.dict())
        return element

    def find(self, search_filter: "Dict[str, Any]" = {}) -> "List[T]":
        """
        Find all elements from DB with a specified search filter
        @param search_filter: Filter for the search
        @return: List[T]
        """
        elements = self._mongo_db_interface.find(search_filter=search_filter)

        return [self._element_type(**element) for element in elements]

    def find_one(self, search_filter: "Dict[str, Any]" = {}) -> "Optional[T]":
        """
        Find one element from DB with a specified search filter
        @param search_filter: Filter for the search
        @return: Optional[T]
        """
        element = self._mongo_db_interface.find_one(search_filter=search_filter)
        if element is None:
            return None

        return self._element_type(**element)

    def update(self, element: "T") -> "T":
        """
        Updates an element in the DB using its id
        @param element: The element to update to (id is used to find the element to update)
        @return: T
        """
        self._mongo_db_interface.update(search_filter={"id": element.id}, updated_document=element.dict())
        return element

    def update_many(self, updated_document: "Dict[str, Any]", search_filter: "Dict[str, Any]" = {}) -> "List[T]":
        """
        Updates multiple elements in the DB using the specified search filter
        :param updated_document: the new values to set for the documents
        :param search_filter: documents to update values for
        :return: List[T]
        """
        self._mongo_db_interface.update(search_filter=search_filter, updated_document=updated_document)
        return self.find(search_filter=search_filter)

    def delete_many(self, search_filter: "Dict[str, Any]", force: bool = False) -> int:
        """
        Deletes elements from the DB
        @param search_filter: The search filter of the elements to delete
        @param force: if True, permanently delete the elements from the DB, otherwise - just mark them as deleted
        @return: Number of records that were deleted
        """
        if force:
            delete_result = self._mongo_db_interface.delete(search_filter=search_filter)
            return delete_result.deleted_count
        update_result = self._mongo_db_interface.update(search_filter=search_filter, updated_document={"deleted": True})
        return update_result.modified_count

    def delete(self, _id: "UUID", force: bool = False) -> bool:
        """
        Deletes an element from the DB
        @param _id: The ID of the element to delete
        @param force: if True, permanently delete the element from the DB, otherwise - just mark it as deleted
        @return: True on success, False otherwise
        """
        search_filter = {"id": _id}
        return self.delete_many(search_filter, force) == 1

    def increment_fields(self, *, search_filter: "Dict[str, Any]", fields: "Dict[str, int]") -> bool:
        return self._mongo_db_interface.increment_field(search_filter=search_filter, increment_query=fields)



from typing import TYPE_CHECKING
from uuid import uuid4

if TYPE_CHECKING:
    from enum import Enum
    from typing import Callable, Dict, List, Type, TypeVar

    T = TypeVar("T", bound=Enum)


def get_random_uuid_string() -> str:
    """
    Returns a uuid4 without dashes.
    """
    return str(uuid4()).replace("-", "")


def get_enum_values(enum_type: "Type[T]", filter_function: "Callable[[T], bool]" = lambda e: True) -> "List[str]":
    """
    Gets all the possible enumerations of an enum.
    """
    return [e.value for e in enum_type if filter_function(e)]


def is_ascii(s: str) -> bool:
    return all(ord(c) < 128 for c in s)


def to_snake_case(string: str) -> str:
    """
    This func gets a string in any case and return it in snakecase.
    Args:
        string (): string to snakecase

    Returns:
        snakecase string
    """
    string = string.replace(" ", "_")
    string = string.replace("-", "_")
    string = string.replace(".", "_")
    snake_case_word = string[0]
    for char in string[1:]:
        if char.isupper() and snake_case_word[-1].islower():
            snake_case_word += f"_{char}"
        else:
            snake_case_word += char
    return snake_case_word.lower()


def to_title_case(string: str) -> str:
    """
    This func gets a string in any case and return it in titlecase.
    Args:
        string (): string to titlecae

    Returns:
        titlecase string
    """
    string = string.replace("_", " ")
    string = string.replace("-", " ")
    string = string.replace(".", " ")
    return_string = string[0].upper()
    for inx, char in enumerate(string[1:]):
        if char.isupper():
            if string[1:][inx - 1].islower():
                return_string += " "
        return_string += char
    return " ".join([word[0].upper() + word[1:] for word in return_string.lower().split(" ")])


def get_all_enum_fields_in_web_form(enum: "Type[Enum]", custom: "Dict[str, str]" = {}) -> "List[str]":
    """
        This function made to serve the platform ui.

    Args:
        enum ():  deci enum class to rescue values from.
        custom (): dict: key - value of field of the enum, value- the value to set the key to. instead of camel case.

    Returns:
        all enum fields transformed to camelCase if not in custom. ex:  'object_detection' to 'objectDetection'
    """
    return get_all_list_fields_in_web_form(list_of_fields=get_enum_values(enum), custom=custom)


def get_all_list_fields_in_web_form(list_of_fields: "List[str]", custom: "Dict[str, str]" = {}) -> "List[str]":
    """
        This function made to serve the platform ui.

    Args:
        list_of_fields (): list of values to convert to web form.
        custom (): dict: key - one of the values of field of the list, value- the value to set the key to. instead of camel case.

    Returns:
        all list fields transformed to camel case if not in custom. ex:  'object_detection' to 'objectDetection'.
    """
    fields = []
    for field in list_of_fields:
        if field in custom.keys():
            field = custom[field]
        else:
            field = to_title_case(field)
        fields.append(field)
    return fields



import logging
import sys
from typing import TYPE_CHECKING

import boto3
from botocore.exceptions import ClientError, ProfileNotFound

if TYPE_CHECKING:
    from typing import Any, Dict, Optional, Type

    from boto3 import Session
    from boto3.resources.base import ServiceResource


class AWSConnector:
    """
    AWSConnector - Connects to AWS using Credentials File or IAM Role
    """

    @staticmethod
    def __get_assumed_role_credentials(aws_role_to_assume: str) -> "Dict[str, str]":
        """
        __get_assumed_role_credentials
            :param aws_role_to_assume: the arn of an AWS role to assume
            :return:
        """
        sts_client = boto3.client("sts")
        assumed_role_object = sts_client.assume_role(
            RoleArn=aws_role_to_assume,
            RoleSessionName=f"{__class__.__name__}-session",  # type: ignore[name-defined]
        )
        return assumed_role_object["Credentials"]

    @staticmethod
    def __create_boto_3_session(
        profile_name: "Optional[str]",
        aws_role_to_assume: "Optional[str]" = None,
    ) -> "Optional[Session]":
        """
        __create_boto_3_session
            :param profile_name:
            :param aws_role_to_assume: (Optional) the arn of an AWS role to assume
            :return:
        """
        current_class_name = __class__.__name__  # type: ignore[name-defined]
        logger = logging.getLogger(current_class_name)

        try:
            if aws_role_to_assume:
                # TRYING TO ASSUME A SPECIFIC ROLE USING DEFAULT CREDENTIALS
                try:
                    temporary_credentials = AWSConnector.__get_assumed_role_credentials(
                        aws_role_to_assume=aws_role_to_assume
                    )
                    session = boto3.Session(
                        aws_access_key_id=temporary_credentials["AccessKeyId"],
                        aws_secret_access_key=temporary_credentials["SecretAccessKey"],
                        aws_session_token=temporary_credentials["SessionToken"],
                    )
                    return session
                except Exception as noCredentialsErrorException:
                    logger.warning(
                        f"[{current_class_name}] - Default Profile/IAM Role Could not assume "
                        f"{aws_role_to_assume} role. This is not an expected behavior."
                        f"{str(noCredentialsErrorException)}"
                    )
            try:
                if profile_name and boto3.session.Session(profile_name=profile_name).get_credentials():
                    # TRY USING A SPECIFIC PROFILE_NAME (USING A CREDENTIALS FILE)
                    logger.info("Trying to connect to AWS using Credentials File with profile_name: " + profile_name)

                    session = boto3.Session(profile_name=profile_name)
                    return session

            except ProfileNotFound as profileNotFoundException:
                logger.debug(
                    "["
                    + current_class_name
                    + "] - Could not find profile name - Trying using Default Profile/IAM Role"
                    + str(profileNotFoundException)
                )

            # TRY USING AN IAM ROLE (OR *DEFAULT* CREDENTIALS - USING A CREDENTIALS FILE)
            logger.info("Trying to connect to AWS using IAM role or Default Credentials")
            session = boto3.Session()
            return session

        except Exception as ex:
            logger.critical(
                "["
                + current_class_name
                + "] - Caught Exception while trying to connect to AWS Credentials Manager "
                + str(ex)
            )
            return None

    @staticmethod
    def get_aws_session(
        profile_name: "Optional[str]",
        aws_role_to_assume: "Optional[str]" = None,
    ) -> "Optional[Session]":
        """
        get_aws_session - Connects to AWS to retrieve an AWS Session
            :param      profile_name: The Config Profile (Environment Name in Credentials file)
            :param      aws_role_to_assume: (Optional) the arn of an AWS role to assume
            :return:    boto3 Session
        """
        current_class_name = __class__.__name__  # type: ignore[name-defined]
        logger = logging.getLogger(current_class_name)

        aws_session = AWSConnector.__create_boto_3_session(
            profile_name=profile_name,
            aws_role_to_assume=aws_role_to_assume,
        )
        if aws_session is None:
            logger.error("Failed to initiate an AWS Session")

        return aws_session

    @staticmethod
    def get_aws_client_for_service_name(
        profile_name: "Optional[str]",
        service_name: str,
        aws_role_to_assume: "Optional[str]" = None,
    ) -> "Any":
        """
        get_aws_client_for_service_name - Connects to AWS to retrieve the relevant Client
            :param      profile_name: The Config Profile (Environment Name in Credentials file)
            :param      service_name: The AWS Service name to get the Client for
            :param      aws_role_to_assume: (Optional) the arn of an AWS role to assume
            :return:    Service client instance
        """
        current_class_name = __class__.__name__  # type: ignore[name-defined]
        logger = logging.getLogger(current_class_name)

        aws_session = AWSConnector.__create_boto_3_session(
            profile_name=profile_name, aws_role_to_assume=aws_role_to_assume
        )
        if aws_session is None:
            logger.error("Failed to connect to AWS client: " + str(service_name))
            return None

        return aws_session.client(service_name=service_name)  # type: ignore[call-overload]

    @staticmethod
    def get_aws_resource_for_service_name(
        profile_name: "Optional[str]",
        service_name: str,
        aws_role_to_assume: "Optional[str]" = None,
    ) -> "Optional[ServiceResource]":
        """
        Connects to AWS to retrieve the relevant Resource (More functionality then Client)
            :param      profile_name: The Config Profile (Environment Name in Credentials file)
            :param      service_name: The AWS Service name to get the Client for
            :param      aws_role_to_assume: (Optional) the arn of an AWS role to assume
            :return:    Service client instance
        """
        current_class_name = __class__.__name__  # type: ignore[name-defined]
        logger = logging.getLogger(current_class_name)

        aws_session = AWSConnector.__create_boto_3_session(
            profile_name=profile_name, aws_role_to_assume=aws_role_to_assume
        )
        if aws_session is None:
            logger.error("Failed to connect to AWS client: " + str(service_name))
            return None

        return aws_session.resource(service_name=service_name)  # type: ignore[call-overload]

    @staticmethod
    def is_client_error(code: str) -> "Type[Exception]":
        e = sys.exc_info()[1]
        if isinstance(e, ClientError) and e.response["Error"]["Code"] == code:
            return ClientError
        return type("NeverEverRaisedException", (Exception,), {})



import logging
import logging.config
from typing import TYPE_CHECKING

from deci_common.auto_logging import AutoLoggerConfig

if TYPE_CHECKING:
    from logging import Logger
    from typing import Optional


def get_logger(logger_name: str) -> "Logger":
    config_dict = AutoLoggerConfig.generate_config_for_module_name(logger_name)
    logging.config.dictConfig(config_dict)
    logger = logging.getLogger(logger_name)
    return logger


class ILogger:
    """
    Provides logging capabilities to the derived class.
    """

    def __init__(self, logger_name: "Optional[str]" = None):
        logger_name = logger_name if logger_name else str(self.__module__)
        self._logger = get_logger(logger_name)



import json
import os
from typing import TYPE_CHECKING

import pkg_resources

from deci_common.environment.environment_config import DEFAULT_LOGGING_LEVEL

if TYPE_CHECKING:
    from typing import Any, Dict, List, Optional


class AutoLoggerConfig:
    """
    A Class for the Automated Logging Config that is created from the JSON config file (auto_logging_conf)
    """

    @staticmethod
    def generate_config_for_module_name(
        module_name: str,
        log_level: str = DEFAULT_LOGGING_LEVEL,
        max_bytes: int = 10485760,
        handlers_list: "Optional[List[str]]" = None,
    ) -> "Dict[str, Any]":
        """
        generate_config_for_module_name - Returns a Config Dict For Logging
            :param module_name:     The Python Module name to create auto_logging for
            :param log_level:       Minimal log level to set for the new auto_logging
            :param max_bytes:       Max size for the log file before rotation starts
            :param handlers_list:    A list specifying the handlers (Console, etc..) - Better Leave Empty or None
            :return: python dict() with the new auto_logging for the module
        """

        # LOADING THE ORIGINAL ROOT CONFIG FILE
        conf_file_name = "auto_logging_conf.json"
        conf_file_path = os.path.join(pkg_resources.resource_filename("deci_common", "/auto_logging/"), conf_file_name)

        with open(conf_file_path, "r") as logging_configuration_file:
            config_dict = json.load(logging_configuration_file)

        if os.environ.get("JSON_LOG_FORMAT") is not None:
            config_dict["handlers"]["console"]["formatter"] = "jsonFormatter"

        # CREATING THE PATH TO THE "HOME" FOLDER WITH THE LOG FILE NAME
        log_file_name = module_name + ".log"
        user_dir = os.path.expanduser(r"~")
        logs_dir_path = os.path.join(user_dir, "deci_logs")

        if not os.path.exists(logs_dir_path):
            try:
                os.mkdir(logs_dir_path)
            except Exception as ex:
                print(
                    "[WARNING] - deci_logs folder was not found and couldn't be created from the code - "
                    "All of the Log output will be sent to Console!" + str(ex)
                )

            # CREATING ONLY A CONSOLE LOG HANDLER
            handlers_list = ["console"]
            logger = {"level": log_level, "handlers": handlers_list, "propagate": False}
            config_dict["loggers"][module_name] = logger

            return config_dict

        log_file_path = os.path.join(logs_dir_path, log_file_name)

        # THE ENTRIES TO ADD TO THE ORIGINAL CONFIGURATION
        handler_name = module_name + "_file_handler"
        file_handler = {
            "class": "logging.handlers.RotatingFileHandler",
            "level": log_level,
            "formatter": "fileFormatter",
            "filename": log_file_path,
            "maxBytes": max_bytes,
            "backupCount": 20,
            "encoding": "utf8",
        }

        # CREATING MULTIPLE HANDLERS - A LOGGING FILE HANDLER AND A CONSOLE HANDLER
        if handlers_list is None or len(handlers_list) == 0:
            handlers_list = ["console", handler_name]

        logger = {"level": log_level, "handlers": handlers_list, "propagate": False}

        # ADDING THE NEW LOGGER ENTRIES TO THE CONFIG DICT
        config_dict["handlers"][handler_name] = file_handler
        config_dict["loggers"][module_name] = logger
        config_dict["root"]["handlers"].append(handler_name)

        return config_dict



import os
import sys
from io import BytesIO, StringIO
from typing import TYPE_CHECKING

import botocore

from deci_common.abstractions.abstract_logger import ILogger
from deci_common.aws_connection import AWSConnector

if TYPE_CHECKING:
    from typing import Any, Dict, List, Optional, Union


class KeyNotExistInBucketError(Exception):
    pass


class DeleteKeyAfterRenamingItFailedError(Exception):
    pass


class S3Connector(ILogger):
    """
    S3Connector - S3 Connection Manager
    """

    def __init__(self, env: "Optional[str]" = None, *, bucket_name: str):
        """
        Initiate a connector to AWS S3 object storage
        :param env: profile name or environment for the s3 client and resource
        :param bucket_name: the name of the bucket to create the connection to
        """
        super().__init__()
        self.bucket_name = bucket_name

        self.s3_client = AWSConnector.get_aws_client_for_service_name(profile_name=env, service_name="s3")
        self.s3_resource = AWSConnector.get_aws_resource_for_service_name(profile_name=env, service_name="s3")

    def check_key_exists(self, s3_key_to_check: str) -> "Optional[bool]":
        """
        check_key_exists - Checks if an S3 key exists
        :param s3_key_to_check:
        :return:
        """
        try:
            self.s3_client.head_object(Bucket=self.bucket_name, Key=s3_key_to_check)
        except botocore.exceptions.ClientError as ex:
            if ex.response["Error"]["Code"] == "404":
                return False
            else:
                self._logger.error(
                    "Failed to check key: " + str(s3_key_to_check) + " existence in bucket" + str(self.bucket_name)
                )
                return None
        else:
            return True

    def get_object_by_etag(self, bucket_relative_file_name: str, etag: str) -> "Optional[object]":
        """
        get_object_by_etag - Gets S3 object by its ETag header if it. exists
        :param bucket_relative_file_name: The name of the file in the bucket (relative)
        :param etag: The ETag of the object in S3
        :return:
        """
        try:
            etag = etag.strip('"')
            s3_object = self.s3_client.get_object(Bucket=self.bucket_name, Key=bucket_relative_file_name, IfMatch=etag)
            return s3_object
        except botocore.exceptions.ClientError as ex:
            if ex.response["Error"]["Code"] == "404":
                return False
            else:
                self._logger.error(
                    "Failed to check ETag: " + str(etag) + " existence in bucket " + str(self.bucket_name)
                )
        return None

    def create_bucket(self, s3_client: "Any" = None) -> bool:
        """
        Creates a bucket with the initialized bucket name.
        :return: The new bucket response
        :raises ClientError: If the creation failed for any reason.
        """
        client = s3_client or self.s3_client
        try:
            # TODO: Change bucket_owner_arn to the company's proper IAM Role
            self._logger.info("Creating Bucket: " + self.bucket_name)
            create_bucket_response = client.create_bucket(ACL="private", Bucket=self.bucket_name)
            self._logger.info(f"Successfully created bucket: {create_bucket_response}")

            # Changing the bucket public access block to be private (disable public access)
            self._logger.debug("Disabling public access to the bucket...")
            client.put_public_access_block(
                PublicAccessBlockConfiguration={
                    "BlockPublicAcls": True,
                    "IgnorePublicAcls": True,
                    "BlockPublicPolicy": True,
                    "RestrictPublicBuckets": True,
                },
                Bucket=self.bucket_name,
            )

            return create_bucket_response
        except botocore.exceptions.ClientError as err:
            self._logger.fatal(f'Failed to create bucket "{self.bucket_name}": {err}')
            raise err

    def check_bucket_exists(
        self,
        bucket_name: "Optional[str]" = None,
        s3_resource: "Any" = None,
    ) -> bool:
        resource = s3_resource or self.s3_resource
        bucket_name = bucket_name if bucket_name else self.bucket_name
        return resource.Bucket(bucket_name) in resource.buckets.all()  # type: ignore[union-attr]

    def delete_bucket(self, s3_resource: "Any" = None) -> bool:
        """
        Deletes a bucket with the initialized bucket name.
        :return: True if succeeded.
        :raises ClientError: If the creation failed for any reason.
        """
        resource = s3_resource or self.s3_resource
        try:
            self._logger.info("Deleting Bucket: " + self.bucket_name + " from S3")
            bucket = resource.Bucket(self.bucket_name)  # type: ignore[union-attr]
            bucket.objects.all().delete()
            bucket.delete()
            self._logger.debug("Successfully Deleted Bucket: " + self.bucket_name + " from S3")
        except botocore.exceptions.ClientError as ex:
            self._logger.fatal(f"Failed to delete bucket {self.bucket_name}: {ex}")
            raise ex
        return True

    def get_object_metadata(self, s3_key: str) -> "Dict[str, Any]":
        try:
            return self.s3_client.head_object(Bucket=self.bucket_name, Key=s3_key)
        except botocore.exceptions.ClientError as ex:
            if ex.response["Error"]["Code"] == "404":
                msg = "[" + sys._getframe().f_code.co_name + "] - Key does not exist in bucket)"
                self._logger.error(msg)
                raise KeyNotExistInBucketError(msg)
            raise ex

    def delete_key(self, s3_key_to_delete: str) -> bool:
        """
        delete_key - Deletes a Key from an S3 Bucket
            :param s3_key_to_delete:
            :return: True/False if the operation succeeded/failed
        """
        try:
            self._logger.debug("Deleting Key: " + s3_key_to_delete + " from S3 bucket: " + self.bucket_name)
            obj_status = self.s3_client.head_object(Bucket=self.bucket_name, Key=s3_key_to_delete)
        except botocore.exceptions.ClientError as ex:
            if ex.response["Error"]["Code"] == "404":
                self._logger.error("[" + sys._getframe().f_code.co_name + "] - Key does not exist in bucket)")
            return False

        if obj_status["ContentLength"]:
            self._logger.debug(
                "[" + sys._getframe().f_code.co_name + "] - Deleting file s3://" + self.bucket_name + s3_key_to_delete
            )
            self.s3_client.delete_object(Bucket=self.bucket_name, Key=s3_key_to_delete)

        return True

    def upload_file_from_stream(self, file: bytes, key: str) -> bool:
        """
        upload_file - Uploads a file to S3 via boto3 interface
                      *Please Notice* - This method is for working with files, not objects
            :param key: The key (filename) to create in the S3 bucket
            :param file: File to upload
            :return True/False if the operation succeeded/failed
        """
        try:
            self._logger.debug("Uploading Key: " + key + " to S3 bucket: " + self.bucket_name)
            _buffer = BytesIO(file)
            self.upload_buffer(key, _buffer)
            return True
        except Exception as ex:
            self._logger.critical(
                "["
                + sys._getframe().f_code.co_name
                + "] - Caught Exception while trying to upload file "
                + str(key)
                + "to S3"
                + str(ex)
            )
            return False

    def upload_file(self, filename_to_upload: str, key: str) -> bool:
        """
        upload_file - Uploads a file to S3 via boto3 interface
                      *Please Notice* - This method is for working with files, not objects
            :param key:                The key (filename) to create in the S3 bucket
            :param filename_to_upload: Filename of the file to upload
            :return True/False if the operation succeeded/failed
        """
        try:
            self._logger.info("Uploading Key: " + key + " to S3 bucket: " + self.bucket_name)

            self.s3_client.upload_file(Bucket=self.bucket_name, Filename=filename_to_upload, Key=key)
            return True

        except Exception as ex:
            self._logger.critical(
                "["
                + sys._getframe().f_code.co_name
                + "] - Caught Exception while trying to upload file "
                + str(filename_to_upload)
                + "to S3"
                + str(ex)
            )
            return False

    def create_folder(self, folder_name: str) -> bool:
        """
        :param folder_name: name of the folder to create.
        :return: bool - true if creation succeeded
        """
        try:
            self._logger.info(f'Creating S3 folder path {self.bucket_name + "/" + folder_name}')

            self.s3_client.put_object(Bucket=self.bucket_name, Key=folder_name + "/")
            return True

        except Exception as ex:
            self._logger.error(
                f"Failed to create folder {folder_name} in bucket {self.bucket_name}, returning False. ", exc_info=ex
            )
            return False

    def rename_key(self, source_key: str, destination_key: str) -> bool:
        self.copy_key(self.bucket_name, source_key=source_key, destination_key=destination_key)
        try:
            delete_success = self.delete_key(s3_key_to_delete=source_key)
            if not delete_success:
                raise
            return True
        except Exception as e:
            raise DeleteKeyAfterRenamingItFailedError(
                f"Failed to remove old key {source_key} after creating new copy with new name {destination_key}. "
            ) from e

    def download_key(self, target_path: str, key_to_download: str) -> bool:
        """
        download_file - Downloads a key from S3 using boto3 to the provided filename
                        Please Notice* - This method is for working with files, not objects
            :param key_to_download:    The key (filename) to download from the S3 bucket
            :param target_path:           Filename of the file to download the content of the key to
            :return:                   True/False if the operation succeeded/failed
        """
        try:
            self._logger.debug("Uploading Key: " + key_to_download + " from S3 bucket: " + self.bucket_name)
            self.s3_client.download_file(Bucket=self.bucket_name, Filename=target_path, Key=key_to_download)
        except botocore.exceptions.ClientError as ex:
            if ex.response["Error"]["Code"] == "404":
                self._logger.error("[" + sys._getframe().f_code.co_name + "] - Key does exist in bucket)")
            else:
                self._logger.critical(
                    "["
                    + sys._getframe().f_code.co_name
                    + "] - Caught Exception while trying to download key "
                    + str(key_to_download)
                    + " from S3 "
                    + str(ex)
                )
            return False

        return True

    def download_keys_by_prefix(
        self,
        s3_bucket_path_prefix: str,
        local_download_dir: str,
        s3_file_path_prefix: str = "",
    ) -> None:
        """
        download_keys_by_prefix - Download all the keys who match the provided in-bucket path prefix and file prefix
            :param s3_bucket_path_prefix:   The S3 "folder" to download from
            :param local_download_dir:      The local directory to download the files to
            :param s3_file_path_prefix:     The specific prefix of the files we want to download
        :return:
        """
        if not os.path.isdir(local_download_dir):
            raise ValueError("[" + sys._getframe().f_code.co_name + "] - Provided directory does not exist")

        paginator = self.s3_client.get_paginator("list_objects")
        prefix = s3_bucket_path_prefix if not s3_file_path_prefix else s3_bucket_path_prefix + "/" + s3_file_path_prefix
        page_iterator = paginator.paginate(Bucket=self.bucket_name, Prefix=prefix)

        for item in page_iterator.search("Contents"):
            if item is not None:
                if item["Key"] == s3_bucket_path_prefix:
                    continue
            key_to_download = item["Key"]
            local_filename = key_to_download.split("/")[-1]
            self.download_key(target_path=local_download_dir + "/" + local_filename, key_to_download=key_to_download)

    def download_file_by_path(self, s3_file_path: str, local_download_dir: str) -> str:
        """
        :param s3_file_path: str - path ot s3 file e.g./ "s3://x/y.zip"
        :param local_download_dir: path to download
        :return:
        """

        if not os.path.isdir(local_download_dir):
            raise ValueError("[" + sys._getframe().f_code.co_name + "] - Provided directory does not exist")

        local_filename = s3_file_path.split("/")[-1]
        self.download_key(target_path=local_download_dir + "/" + local_filename, key_to_download=s3_file_path)
        return local_filename

    def empty_folder_content_by_path_prefix(self, s3_bucket_path_prefix: str) -> "List[Dict[str, Any]]":
        """
        empty_folder_content_by_path_prefix - Deletes all the files in the specified bucket path
            :param s3_bucket_path_prefix: The "folder" to empty
            :returns: Errors list
        """
        paginator = self.s3_client.get_paginator("list_objects")
        page_iterator = paginator.paginate(Bucket=self.bucket_name, Prefix=s3_bucket_path_prefix)

        files_dict_to_delete: "Dict[str, Any]" = dict(Objects=[])
        errors_list: "List[Dict[str, Any]]" = []

        for item in page_iterator.search("Contents"):
            if item is not None:
                if item["Key"] == s3_bucket_path_prefix:
                    continue
                files_dict_to_delete["Objects"].append(dict(Key=item["Key"]))

                # IF OBJECTS LIMIT HAS BEEN REACHED, FLUSH
                if len(files_dict_to_delete["Objects"]) >= 1000:
                    self._delete_files_left_in_list(errors_list, files_dict_to_delete)
                    files_dict_to_delete = dict(Objects=[])

        # DELETE THE FILES LEFT IN THE LIST
        if len(files_dict_to_delete["Objects"]):
            self._delete_files_left_in_list(errors_list, files_dict_to_delete)

        return errors_list

    def _delete_files_left_in_list(
        self,
        errors_list: "List[Dict[str, Any]]",
        files_dict_to_delete: "Dict[str, Any]",
    ) -> None:
        try:
            s3_response = self.s3_client.delete_objects(Bucket=self.bucket_name, Delete=files_dict_to_delete)
        except Exception as ex:
            self._logger.critical(
                "["
                + sys._getframe().f_code.co_name
                + "] - Caught Exception while trying to delete keys "
                + "from S3 "
                + str(ex)
            )
        if "Errors" in s3_response:
            errors_list.append(s3_response["Errors"])

    def upload_buffer(self, new_key_name: str, buffer_to_write: "Union[StringIO, BytesIO]") -> None:
        """
        Uploads a buffer into a file in S3 with the provided key name.
        :bucket: The name of the bucket
        :new_key_name: The name of the file to create in s3
        :buffer_to_write: A buffer that contains the file contents.
        """
        self.s3_resource.Object(self.bucket_name, new_key_name).put(Body=buffer_to_write.getvalue())  # type: ignore[union-attr]

    def upload_empty_file(self, key_name: str) -> None:
        """
        Uploads an empty buffer to S3 bucket in the provided key name
        :param key_name: the key to upload an empty file to
        :return: None
        """
        self.upload_buffer(new_key_name=key_name, buffer_to_write=StringIO())

    def list_bucket_objects(self, prefix: "Optional[str]" = None) -> "List[Dict[str, Any]]":
        """
        Gets a list of dictionaries, representing files in the S3 bucket that is passed in the constructor (self.bucket).
        :param prefix: A prefix filter for the files names.
        :return: the objects, dict as received from botocore.
        """
        paginator = self.s3_client.get_paginator("list_objects")
        if prefix:
            page_iterator = paginator.paginate(Bucket=self.bucket_name, Prefix=prefix)
        else:
            page_iterator = paginator.paginate(Bucket=self.bucket_name)

        bucket_objects = []
        for item in page_iterator.search("Contents"):
            if not item or item["Key"] == self.bucket_name:
                continue
            bucket_objects.append(item)
        return bucket_objects

    def create_presigned_upload_url(
        self,
        object_name: str,
        fields: "Any" = None,
        conditions: "Any" = None,
        expiration: int = 3600,
        override: bool = False,
    ) -> "Dict[str, Any]":
        """Generate a presigned URL S3 POST request to upload a file
        :param object_name: string
        :param fields: Dictionary of prefilled form fields
        :param conditions: List of conditions to include in the policy
        :param expiration: Time in seconds for the presigned URL to remain valid
        :param override: Allows disregarding the existence of an object in S3
        :return: Dictionary with the following keys:
            url: URL to post to
            fields: Dictionary of form fields and values to submit with the POST request
        """
        # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/s3-presigned-urls.html#generating-a-presigned-url-to-upload-a-file
        if not override:
            file_already_exist = self.check_key_exists(object_name)
            if file_already_exist:
                raise FileExistsError(f"The key {object_name} already exists in bucket {self.bucket_name}")

        response = self.s3_client.generate_presigned_post(
            self.bucket_name, object_name, Fields=fields, Conditions=conditions, ExpiresIn=expiration
        )
        return response

    def create_presigned_download_url(
        self,
        object_name: str,
        expiration: int = 3600,
        download_name: "Optional[str]" = None,
    ) -> str:
        """Generate a presigned URL S3 Get request to download a file
        :param object_name: string
        :param expiration: Time in seconds for the presigned URL to remain valid
        :param download_name: If given the downloaded file will be renamed to this name
        :return: URL encoded with the credentials in the query, to be fetched using any HTTP client.
        """
        # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/s3-presigned-urls.html
        if download_name is None:
            # overriding the file name if specified
            download_name = object_name
        response = self.s3_client.generate_presigned_url(
            "get_object",
            Params={
                "Bucket": self.bucket_name,
                "Key": object_name,
                "ResponseContentDisposition": f'attachment; filename="{download_name}"',
            },
            ExpiresIn=expiration,
        )
        return response

    @staticmethod
    def convert_content_length_to_mb(content_length: int) -> float:
        return round(float(f"{content_length / (1e+6):2f}"), 2)

    def copy_key(self, destination_bucket_name: str, source_key: str, destination_key: str) -> bool:
        self._logger.info(
            f"Copying the bucket object {self.bucket_name}/{source_key} to {destination_bucket_name}/{destination_key}"
        )
        copy_source = {"Bucket": self.bucket_name, "Key": source_key}

        # Copying the key
        bucket = self.s3_resource.Bucket(destination_bucket_name)  # type: ignore[union-attr]
        bucket.copy(copy_source, destination_key)
        return True

    def change_lifecycle_configuration(self, lifecycle_configuration: "Dict[str, Any]") -> None:
        self.s3_client.put_bucket_lifecycle_configuration(
            Bucket=self.bucket_name, LifecycleConfiguration=lifecycle_configuration
        )

    def set_expiration_for_prefix(self, prefix: str, days_to_expiration: int) -> None:
        lifecycle_configuration = {
            "Rules": [
                {
                    "Expiration": {"Days": days_to_expiration},
                    "Filter": {"Prefix": prefix},
                    "ID": f"Expiration for {self.bucket_name}/{prefix}",
                    "Status": "Enabled",
                }
            ]
        }
        self.change_lifecycle_configuration(lifecycle_configuration=lifecycle_configuration)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['deci_common',
 'deci_common.abstractions',
 'deci_common.auto_logging',
 'deci_common.aws_connection',
 'deci_common.data_connection',
 'deci_common.data_interfaces',
 'deci_common.data_types',
 'deci_common.data_types.enum',
 'deci_common.environment']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1,<2',
 'botocore>=1,<2',
 'cryptography==3.4.7',
 'gdown>=4,<5',
 'numpy>=1.19,<2.0',
 'pydantic>=1.9,<2.0',
 'pymongo==4.1.1',
 'python-json-logger==2.0.2',
 'python-magic>=0,<1',
 'stringcase>=1,<2']

setup_kwargs = {
    'name': 'deci-common',
    'version': '11.1.0',
    'description': "A package for Deci's common data types, connections and interfaces",
    'long_description': '# deci-common\nCommon data types, connections and interfaces\n',
    'author': 'Deci AI',
    'author_email': 'rnd@deci.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



import os
import os.path
import socket
import tempfile
import urllib.error
import zipfile
from typing import Any, Optional
from urllib.parse import urlparse
from urllib.request import urlretrieve
from uuid import uuid4

import gdown  # type: ignore[import]
import requests
from cryptography.fernet import Fernet, InvalidToken

from deci_common.data_types.model_errors import BadModelFileError, BadModelFileReason

DOWNLOAD_MODEL_TIMEOUT_SECONDS = 5 * 60


class FileDownloadFailedException(Exception):
    pass


class FileCreationFailedException(Exception):
    pass


class FileIsNotEncryptedError(Exception):
    pass


class FilesDataInterface:
    @staticmethod
    def add_directory_to_archive(zipf: zipfile.ZipFile, path: str, zip_root: str = ".") -> None:
        for root, dirs, files in os.walk(path):
            for file in files:
                file_full_path = os.path.join(root, file)
                zip_root_path = os.path.join(path, zip_root)
                relative_path_from_zip_root = os.path.relpath(file_full_path, zip_root_path)
                zipf.write(filename=file_full_path, arcname=relative_path_from_zip_root)

    @staticmethod
    def archive_directory(path: str, archive_path: str, zip_root: str = "../../../deci-common/deci_common") -> str:
        zipf = zipfile.ZipFile(archive_path, "w", zipfile.ZIP_DEFLATED)
        FilesDataInterface.add_directory_to_archive(zipf=zipf, path=path, zip_root=zip_root)
        zipf.close()

        return archive_path

    @staticmethod
    def download_file_to_memory(file_url: str) -> Any:
        """
        Download a File.
        Args:
            file_url (): The url to download file from

        Returns:
            str contains the binary memory file
        """
        req_for_file = requests.get(file_url, stream=True)
        file_object_from_req = req_for_file.raw
        file = file_object_from_req.read()
        return file

    @staticmethod
    def create_temporary_file(content: str) -> str:
        try:
            fd, path = tempfile.mkstemp()
            with os.fdopen(fd, "w") as tmp:
                tmp.write(content)
        except Exception as e:
            msg = f"Failed to create temporary file with error {e}"
            raise FileCreationFailedException(msg) from e
        return path

    @staticmethod
    def download_temporary_file(
        file_url: str,
        temp_file_prefix: str = "",
        temp_file_suffix: str = "",
        timeout_seconds: Optional[int] = DOWNLOAD_MODEL_TIMEOUT_SECONDS,
    ) -> str:
        """
        Downloads a file to /tempfile.gettempdir/.

        Args:
        :param file_url: The url to downlaod from
        :param temp_file_prefix: The predix to add to the file in the temporary folder
        :param temp_file_suffix: The suffix to add to the file in the temporary folder
        :param timeout_seconds: The timeout in seconds to pass to urllib.setdefaulttimeout. Pass None for no timeout.
        The default timeout is DOWNLOAD_MODEL_TIMEOUT_SECONDS;
        :return: The file's path.
        :raises: FileDownloadFailedException
        """
        tmp_dir = tempfile.gettempdir()
        file_name = temp_file_prefix + str(uuid4()) + temp_file_suffix
        file_path = tmp_dir + os.path.sep + file_name

        # GET THE URL'S FILE EXTENSION
        parsed_url = urlparse(file_url)
        _, file_extension = os.path.splitext(parsed_url.path)

        if file_extension:
            file_path = file_path + file_extension

        # Proprietary downloads
        if "drive.google.com" in file_url:
            FilesDataInterface._download_google_drive_file(file_url, file_path)
        else:
            # TODO: Use requests with stream and limit the file size and timeouts.
            socket.setdefaulttimeout(timeout_seconds)
            try:
                urlretrieve(file_url, file_path)  # nosec: B310
            except urllib.error.ContentTooShortError as ex:
                raise FileDownloadFailedException("File download did not finish correctly " + str(ex))

        return file_path

    @staticmethod
    def save_bytes_to_file(file_bytes: bytes, file_name: Optional[str] = None, mode: str = "wb") -> str:
        """
        Writes a buffer to a local file.
        :param file_bytes: The bytes to write.
        :param file_name: The name of the file to create. Defaults to /tempfile.gettempdir/{uuid4} if not specified.
        :param mode: The write open mode.
        """
        file_name = file_name or tempfile.gettempdir() + os.path.sep + str(uuid4())
        with open(file_name, mode) as new_file:
            new_file.write(file_bytes)
        return file_name

    @staticmethod
    def _download_google_drive_file(google_drive_url: str, destination_file_name: str) -> None:
        """
        Wraps gdown package to download files
        :param google_drive_url: The google drive file url.
        :param destination_file_name: The full path the file to save the google drive file into.
        :return:
        """
        try:
            gdown.download(google_drive_url, destination_file_name, quiet=False)
        except Exception as ex:
            raise FileDownloadFailedException("File download did not finish correctly " + str(ex))

    @staticmethod
    def encrypt_file(file: bytes, key: bytes) -> bytes:
        """
        This method get a file and a key and encrypt the file with the key.
        Args:
            file (): file to encrypt - byte stream : open(file,'rb'), or str.encode()
            key (): byte str : Fernet.gen_key()

        Returns:
            str contains the cipher file.
        """
        try:
            # Instantiate the object with your key.
            f = Fernet(key)
            # Pass your bytes type message into encrypt.
            ciphertext = f.encrypt(file)
            return ciphertext
        except Exception as e:
            msg = f"Failed to encrypt file {file!r}, err: {e}"
            raise Exception(msg) from e

    @staticmethod
    def decrypt_file(file: bytes, key: bytes) -> bytes:
        """
        This method get a file and a key and decrypt the file with the key.
        Args:
            file (): file to decrypt - encrypted file: FilesDataInterface.encrypt_file()
            key (): byte str : Fernet.gen_key()
        Returns:
            str contains the cipher file.
        """
        try:
            # Load the private key from a file.
            # Instantiate Fernet on the recip system.
            f = Fernet(key)
            # Decrypt the message.
            cleartext = f.decrypt(file)
            return cleartext
        except (InvalidToken, TypeError) as e:
            raise FileIsNotEncryptedError(f"Failed to decrypt file {file!r}, probably it not encrypted.") from e
        except Exception as e:
            msg = f"Failed to decrypt file {file!r}, err: {e}"
            raise Exception(msg) from e

    @staticmethod
    def unzip_tf2_saved_model_checkpoint_file(tf2_saved_model_checkpoint_file_path: str) -> str:
        """
        Unzip a TF2 Saved Model proto to a temp location
            :param tf2_saved_model_checkpoint_file_path:
            :return: The path to the unzipped saved model directory
        """
        try:
            with zipfile.ZipFile(tf2_saved_model_checkpoint_file_path, "r") as zipObj:
                destination_folder = tempfile.mkdtemp(prefix="tf2_saved_model_checkpoints_")
                zipObj.extractall(destination_folder)
        except Exception as e:
            raise BadModelFileError(reason=BadModelFileReason.CORRUPTED_ZIP_FILE) from e

        # IF UNZIPPED FOLDER CONTAINS ONLY A SINGLE FOLDER, MOVE ALL FILES INTO THE UNZIP FOLDER (IN CASE
        # THE ZIP FILE CONTAINED A SINGLE FOLDER)
        if len(os.listdir(destination_folder)) == 1 and os.path.isdir(
            os.path.join(destination_folder, os.listdir(destination_folder)[0])
        ):
            internal_dir = os.path.join(destination_folder, os.listdir(destination_folder)[0])
            for f in os.listdir(internal_dir):
                os.rename(os.path.join(internal_dir, f), os.path.join(destination_folder, f))
            os.removedirs(internal_dir)

        return str(destination_folder)



from enum import Enum


class DeepLearningTask(str, Enum):
    CLASSIFICATION = "classification"
    SEMANTIC_SEGMENTATION = "semantic_segmentation"
    OBJECT_DETECTION = "object_detection"
    NLP = "nlp"
    TEXT_RECOGNITION = "text_recognition"
    OTHER = "other"


class DeepLearningTaskLabel(str, Enum):
    CLASSIFICATION = "Classification"
    SEMANTIC_SEGMENTATION = "Semantic Segmentation"
    OBJECT_DETECTION = "Object Detection"
    NLP = "NLP"
    TEXT_RECOGNITION = "Text Recognition"
    OTHER = "Other"
